/**
 * Custom NagVis Right-Click Context Menu
 * ---------------------------------------
 * This script enables a right-click context menu on the map background
 * that replicates the "Edit Map" dropdown options from the top header.
 *
 */

document.addEventListener('contextmenu', function (event) {
    const insideNagVisObject = event.target.closest('.context, .icon, .element');
    if (insideNagVisObject) return;

    event.preventDefault();
    showClonedEditMapMenu(event.pageX, event.pageY);
});

document.addEventListener('click', function () {
    const menu = document.getElementById('custom-edit-map-menu');
    if (menu) menu.remove();
});

function showClonedEditMapMenu(x, y) {
    const existing = document.getElementById('custom-edit-map-menu');
    if (existing) existing.remove();

    const original = document.getElementById('map-ddcontent');
    if (!original) {
        console.warn('Edit Map menu not found.');
        return;
    }

    const cloneWrapper = document.createElement('div');
    cloneWrapper.id = 'custom-edit-map-menu-wrapper';
    cloneWrapper.style.position = 'absolute';
    cloneWrapper.style.left = x + 'px';
    cloneWrapper.style.top = y + 'px';
    cloneWrapper.style.zIndex = 10000;

    const clone = original.cloneNode(true);
    clone.id = 'custom-edit-map-menu';

    // Style menu container
    clone.style.display = 'block';
    clone.style.backgroundColor = '#fbfbfb';
    clone.style.boxShadow = '0 4px 12px rgba(0,0,0,0.25)';
    clone.style.borderRadius = '6px';
    clone.style.padding = '6px 0';
    clone.style.minWidth = '220px';
    clone.style.listStyle = 'none';
    clone.style.fontFamily = '"Helvetica Neue", Helvetica, Arial, sans-serif';

    // Style individual list items and links
    const liList = clone.querySelectorAll('li');
    liList.forEach((li, index) => {
        li.style.border = 'none';
        li.style.borderBottom = 'none';

        if (li.classList.contains('spacer')) {
            li.innerHTML = '';
            li.style.width = '203px';
            const hr = document.createElement('hr');
            hr.style.border = '0';
            hr.style.height = '1px';
            hr.style.margin = '6px auto';
            hr.style.width = '90%';
            hr.style.backgroundColor = '#e5e5e5';
            li.appendChild(hr);
            return;
        }

        const link = li.querySelector('a');
        if (link) {
            link.style.color = '#43B3CF';
            link.style.textDecoration = 'none';
            link.style.display = 'block';
            link.style.padding = '6px 12px';
            link.style.fontSize = '13px';
            link.style.whiteSpace = 'nowrap';
            link.style.backgroundColor = '#FBFBFB';
            link.style.borderLeft = '1px solid #e5e5e5';
            link.style.borderRight = '1px solid #e5e5e5';
            link.style.width = '193px';
            link.style.lineHeight = '20px';

            link.addEventListener('mouseenter', () => {
                link.style.backgroundColor = '#ffffff';
                link.style.color = '#789D2B';
            });
            link.addEventListener('mouseleave', () => {
                link.style.backgroundColor = '#FBFBFB';
                link.style.color = '#43B3CF';
            });
        }

        if (index === liList.length - 1) {
            li.style.marginTop = '10px';
        }
    });

    // Handle submenus
    const submenus = clone.querySelectorAll('ul[id$="-ddcontent"]');
    submenus.forEach((submenu) => {
        const originalId = submenu.id;
        const uniqueId = originalId + '_ctx';
        submenu.id = uniqueId;
        submenu.style.display = 'none';
        submenu.style.position = 'absolute';
        submenu.style.left = 'calc(100% - 4px)';
        submenu.style.top = '-6px';
        submenu.style.marginLeft = '4px';
        submenu.style.boxShadow = 'none';
        submenu.style.borderRadius = '4px';
        submenu.style.padding = '4px 0';
        submenu.style.minWidth = '200px';

        const headerId = originalId.replace('content', 'header');
        const header = clone.querySelector(`#${headerId}`);
        if (header) {
            header.id = headerId + '_ctx';
            const li = header.closest('li');

            if (li) {
                li.addEventListener('mouseenter', () => {
                    document.querySelectorAll('.subdropdown ul').forEach(ul => ul.style.display = 'none');
                    const sub = document.getElementById(uniqueId);
                    if (sub) sub.style.display = 'block';
                });
                li.addEventListener('mouseleave', () => {
                    const sub = document.getElementById(uniqueId);
                    if (sub) sub.style.display = 'none';
                });
            }
        }
    });

    cloneWrapper.appendChild(clone);
    document.body.appendChild(cloneWrapper);
}
