<?php

/*****************************************************************************
 *
 * GlobalBackendndomy.php - backend class for handling object and state
 *                           information stored in the NDO database.
 *
 * Copyright (c) 2004-2016 NagVis Project (Contact: info@nagvis.org)
 *
 * License:
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 as
 * published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 *****************************************************************************/

class GlobalBackendndomy extends GlobalBackendPDO {
    public function driverName() {
        return 'mysql';
    }

    /**
     * Sends acknowledgement command to nagios core
     * ADDED BY NAGIOS ENTERPRISES
     */
    public function actionAcknowledge($what, $spec, $comment, $sticky, $notify, $persist, $user)
    {
        $base = realpath(dirname(__FILE__).'/../../../../../');
        define('CFG_ONLY', true);
        require_once($base.'/nagiosxi/html/config.inc.php');

        if ($what == 'host') {
            $what = 'HOST';
        } else if ($what == 'service') {
            $what = 'SVC';
        }

        $sticky  = $sticky ? '2' : '0';
        $notify  = $notify ? '1' : '0';
        $persist = $notify ? '1' : '0';

        $cmd = 'ACKNOWLEDGE_'.$what.'_PROBLEM;'.$spec.';'.$sticky.';'.$notify.';'.$persist.';'.$user.';'.$comment;

        $command_file = $cfg['component_info']['nagioscore']['cmd_file'];

        if (file_exists($command_file) == false) {
            $output = "ERROR: Command file does not exist";
            return false;
        }

        if (($fp = @fopen($command_file, "w")) == false) {
            $output = "ERROR: Could not open command file";
            return false;
        }
        $now = time();
        $command = sprintf("[%lu] %s\n", $now, $cmd);
        fputs($fp, $command);
        fclose($fp);
    }
    
    /**
     * PUBLIC getHostState()
     *
     * Returns the Nagios state and additional information for the requested host
     */
    public function getHostState($objects, $options, $filters, $isMemberQuery = false) {
        $arrReturn = Array();

        $filter = $this->parseFilter($objects, $filters, 'o', 'o', $isMemberQuery, false, HOST_QUERY);
        $QUERYHANDLE = $this->DB->query('SELECT
            o.object_id, alias, display_name, address, o.name1,
            has_been_checked,
            last_hard_state,
            UNIX_TIMESTAMP(last_hard_state_change) AS last_hard_state_change,
            UNIX_TIMESTAMP(last_state_change) AS last_state_change,
            current_state,
            output, perfdata,
            h.notes,
            problem_has_been_acknowledged,
            UNIX_TIMESTAMP(last_check) AS last_check, UNIX_TIMESTAMP(next_check) AS next_check,
            hs.state_type, hs.current_check_attempt, hs.max_check_attempts, hs.check_command,
            UNIX_TIMESTAMP(dh.scheduled_start_time) AS downtime_start, UNIX_TIMESTAMP(dh.scheduled_end_time) AS downtime_end,
            dh.author_name AS downtime_author, dh.comment_data AS downtime_data,
            GROUP_CONCAT(CONCAT(
                    REPLACE(cv.varname, \',\', \'\'),
                    \'=\',
                    REPLACE(cv.varvalue, \',\', \'\')
                )
            ) AS custom_vars
        FROM
            '.$this->dbPrefix.'hosts AS h,
            '.$this->dbPrefix.'objects AS o
        LEFT JOIN
            '.$this->dbPrefix.'hoststatus AS hs
            ON hs.host_object_id=o.object_id
        LEFT JOIN
            '.$this->dbPrefix.'scheduleddowntime AS dh
            ON dh.object_id=o.object_id AND NOW()>dh.scheduled_start_time AND NOW()<dh.scheduled_end_time
        LEFT JOIN
            '.$this->dbPrefix.'customvariables AS cv
            ON cv.object_id=o.object_id
        WHERE
            (o.objecttype_id=1 AND ('.$filter['query'].')
            AND o.instance_id=:instance)
            AND (h.config_type=:configType AND h.instance_id=:instance AND h.host_object_id=o.object_id)
            AND (o.is_active=1)
        GROUP BY o.object_id
	    ', array_merge(
            $filter['params'],
            array('instance' => $this->dbInstanceId, 'configType' => $this->objConfigType)));

        while($data = $QUERYHANDLE->fetch()) {

            // If there is a downtime for this object, save the data
            $in_downtime = 0;
            $dt_details = array(null, null, null, null);
            if($this->DB->is_nonnull_int($data['downtime_start'])) {
                $in_downtime = 1;
                $dt_details = array($data['downtime_author'], $data['downtime_data'],
                                    $data['downtime_start'], $data['downtime_end']);
            }

            /**
                * Only recognize hard states. There was a discussion about the implementation
                * This is a new handling of only_hard_states. For more details, see:
                * http://www.nagios-portal.de/wbb/index.php?page=Thread&threadID=8524
                *
                * Thanks to Andurin and fredy82
                */
            if($options & 1)
                if(!$this->DB->eq_int($data['state_type'], 0))
                    $data['current_state'] = $data['current_state'];
                else
                    $data['current_state'] = $data['last_hard_state'];

            $acknowledged = 0;

            if($this->DB->null_or_eq_int($data['has_been_checked'], 0) || !$this->DB->is_nonnull_int($data['current_state'])) {
                $state = UNCHECKED;
                $output = l('hostIsPending', Array('HOST' => $data['name1']));
            } elseif($this->DB->eq_int($data['current_state'], 0)) {
                // Host is UP
                $state = UP;
                $output = $data['output'];
            } else {
                // Host is DOWN/UNREACHABLE/UNKNOWN

                $acknowledged = intval($data['problem_has_been_acknowledged']);

                // Store state and output in array
                switch(intval($data['current_state'])) {
                    case 1:
                        $state = DOWN;
                        $output = $data['output'];
                    break;
                    case 2:
                        $state = UNREACHABLE;
                        $output = $data['output'];
                    break;
                    case 3:
                        $state = UNKNOWN;
                        $output = $data['output'];
                    break;
                    default:
                        $state = UNKNOWN;
                        $output = 'GlobalBackendndomy::getHostState: Undefined state!';
                    break;
                }
            }

            // takes the concatenated string an breaks it down into an array
            if (isset($data['custom_vars'])){
                $vars = explode(',', $data['custom_vars']);
                foreach ($vars as $var){
                    list($key, $value) = explode('=', $var);
                    if (isset($key) && isset($value)){
                        $customVarsArray[$key] = $value;
                    }
                }
                $data['custom_vars'] = $customVarsArray;
            }

            $arrReturn[$data['name1']] = array(
                $state,
                $output,
                $acknowledged,
                $in_downtime,
                0, // staleness
                $data['state_type'],
                $data['current_check_attempt'],
                $data['max_check_attempts'],
                $data['last_check'],
                $data['next_check'],
                $data['last_hard_state_change'],
                $data['last_state_change'],
                $data['perfdata'],
                $data['display_name'],
                $data['alias'],
                $data['address'],
                $data['notes'],
                $data['check_command'],
                $data['custom_vars'],
                $dt_details[0], // downtime author
                $dt_details[1], // downtime comment
                $dt_details[2], // downtime start
                $dt_details[3], // downtime end
            );
        }

        return $arrReturn;
    }
}
?>
